/*
 * collocate.h
 *
 * Copyright (c) 1995 by Jim Goodman
 *
 * ALL RIGHTS RESERVED.
 *
 * This source file is part of the SCI protocol module for the Wisconsin
 * Wind Tunnel, written by Alain K\"agi.
 *
 */

#ifndef _collocate_h
#define _collocate_h

#include "protocol.h"

/*
 *  col.lo.ca.tion n  \.kal-*-'ka-sh*n\
 *     : the act or result of placing or arranging together; specif 
 *         : a noticeable arrangement or conjoining of linguistic 
 *         elements (as words)
 *
 * two macros:
 *
 *  COLLOCATE_BEGIN(NAME)
 *
 * 	Placed between the lock and the data it protects.  If
 * 	collocation is off, an array of integers will ensure that lock
 * 	and data sit in different cache lines.
 *
 * COLLOCATE_END(NAME, OFF_SIZE, ON_SIZE)
 *
 * 	Placed after the data that the lock protects.  It can be used
 * 	to ensure that the protected data and what follows sit in
 * 	different cache lines.  The last two parameters correspond to
 * 	the size of the padding (in number of integers).  The second
 * 	parameter is the size of the padding if collocation is off,
 * 	and the last parameter is the size of the padding if
 * 	collocation is on.  Beware of alignment restrictions.
 */

#ifdef COLLOCATION

#define COLLOCATE_BEGIN(NAME)
#define COLLOCATE_END(NAME, OFF_SIZE, ON_SIZE)				      \
				int NAME[ON_SIZE]

#else

#define COLLOCATE_BEGIN(NAME)	int NAME[BLOCK_SIZE/4 - 1]
#define COLLOCATE_END(NAME, OFF_SIZE, ON_SIZE)				      \
				int NAME[OFF_SIZE]

#endif COLLOCATION


#endif _collocate_h



