/*
 * lock.h
 *
 * Copyright (c) 1995 by Jim Goodman
 *
 * ALL RIGHTS RESERVED.
 *
 * This source file is part of the SCI protocol module for the Wisconsin
 * Wind Tunnel, written by Alain K\"agi.
 *
 */

#ifndef _lock_h
#define _lock_h

#include "protocol.h"
#include "inline.h"


#if !defined(ENTRY_CONSISTENCY) || !defined(COLLOCATION)

#define LOCKDEC(lockname)		___LOCKDEC(lockname)
#define LOCKINIT(lockname)		___LOCKINIT(lockname)
#define LOCK(lockname)			___LOCK(lockname)
#define UNLOCK(lockname)		___UNLOCK(lockname)
#define LOCK_ENTRY(lockname)		LOCK(lockname)
#define UNLOCK_ENTRY(lockname)		UNLOCK(lockname)
#define LOCK_PREFETCH(lockname)		___LOCK_PREFETCH(lockname)
#define LOCK_UNPREFETCH(lockname)	___LOCK_UNPREFETCH(lockname)

#else

#define LOCKDEC(lockname)		___LOCKDEC(lockname)
#define LOCKINIT(lockname)		___LOCKINIT(lockname)
#define LOCK(lockname)			___LOCK(lockname)
#define UNLOCK(lockname)		___UNLOCK(lockname)
#define LOCK_ENTRY(lockname)		___LOCK_ENTRY(lockname)
#define UNLOCK_ENTRY(lockname)		___UNLOCK_ENTRY(lockname)
#define LOCK_PREFETCH(lockname)		___LOCK_PREFETCH(lockname)
#define LOCK_UNPREFETCH(lockname)	___LOCK_UNPREFETCH(lockname)

#endif

#ifndef PREFETCH

#define ___LOCKDEC(lockname)		__LOCKDEC(lockname)
#define ___LOCKINIT(lockname)		__LOCKINIT(lockname)
#define ___LOCK(lockname)		__LOCK(lockname)
#define ___UNLOCK(lockname)		__UNLOCK(lockname)
#define ___LOCK_ENTRY(lockname)		__LOCK_ENTRY(lockname)
#define ___UNLOCK_ENTRY(lockname)	__UNLOCK_ENTRY(lockname)
#define ___LOCK_PREFETCH(lockname)
#define ___LOCK_UNPREFETCH(lockname)

#else

#define ___LOCKDEC(lockname)		__LOCKDEC(lockname)
#define ___LOCKINIT(lockname)		__LOCKINIT(lockname)
#define ___LOCK(lockname)		__LOCK(lockname)
#define ___UNLOCK(lockname)		__UNLOCK(lockname)
#define ___LOCK_ENTRY(lockname)		__LOCK_ENTRY(lockname)
#define ___UNLOCK_ENTRY(lockname)	__UNLOCK_ENTRY(lockname)
#define ___LOCK_PREFETCH(lockname)	__LOCK_PREFETCH(lockname)
#define ___LOCK_UNPREFETCH(lockname)	__LOCK_UNPREFETCH(lockname)

#endif

#ifndef LOCKSTATS

#define __LOCKDEC(lockname)		_LOCKDEC(lockname)
#define __LOCKINIT(lockname)		_LOCKINIT(lockname)
#define __LOCK(lockname)		_LOCK(lockname)
#define __UNLOCK(lockname)		_UNLOCK(lockname)
#define __LOCK_ENTRY(lockname)		_LOCK_ENTRY(lockname)
#define __UNLOCK_ENTRY(lockname)	_UNLOCK_ENTRY(lockname)
#define __LOCK_PREFETCH(lockname)	_LOCK_PREFETCH(lockname)
#define __LOCK_UNPREFETCH(lockname)	_LOCK_UNPREFETCH(lockname)

#else

/* disable unlock stats */
#define LOCKSTATS_LOCK_IN(lockname) \
				wwt_lockstats_lock_in(lockname)
#define LOCKSTATS_LOCK_OUT(lockname) \
				wwt_lockstats_lock_out(lockname)
#define LOCKSTATS_UNLOCK_IN(lockname) \
				wwt_lockstats_unlock_in(lockname)
#define LOCKSTATS_UNLOCK_OUT(lockname) \
				wwt_lockstats_unlock_out(lockname)
#define LOCKSTATS_LOCK_ENTRY_IN(lockname) \
				wwt_lockstats_lock_entry_in(lockname)
#define LOCKSTATS_LOCK_ENTRY_OUT(lockname) \
				wwt_lockstats_lock_entry_out(lockname)
#define LOCKSTATS_UNLOCK_ENTRY_IN(lockname) \
				wwt_lockstats_unlock_entry_in(lockname)
#define LOCKSTATS_UNLOCK_ENTRY_OUT(lockname) \
				wwt_lockstats_unlock_entry_out(lockname)
#define LOCKSTATS_PREFETCH(lockname) \
				wwt_lockstats_prefetch(lockname)
#define LOCKSTATS_UNPREFETCH(lockname) \
				wwt_lockstats_unprefetch(lockname)

#define __LOCKDEC(lockname)		_LOCKDEC(lockname)
#define __LOCKINIT(lockname)		_LOCKINIT(lockname)
#define __LOCK(lockname)						\
				LOCKSTATS_LOCK_IN(lockname);		\
				_LOCK(lockname);			\
				LOCKSTATS_LOCK_OUT(lockname)
#define __UNLOCK(lockname)						\
				LOCKSTATS_UNLOCK_IN(lockname);		\
				_UNLOCK(lockname);			\
				LOCKSTATS_UNLOCK_OUT(lockname)
#define __LOCK_ENTRY(lockname)						\
				LOCKSTATS_LOCK_ENTRY_IN(lockname);	\
				_LOCK_ENTRY(lockname);			\
				LOCKSTATS_LOCK_ENTRY_OUT(lockname)
#define __UNLOCK_ENTRY(lockname)					\
				LOCKSTATS_UNLOCK_ENTRY_IN(lockname);	\
				_UNLOCK_ENTRY(lockname);		\
				LOCKSTATS_UNLOCK_ENTRY_OUT(lockname)
#define __LOCK_PREFETCH(lockname)					\
				LOCKSTATS_PREFETCH(lockname);		\
				_LOCK_PREFETCH(lockname)
#define __LOCK_UNPREFETCH(lockname)					\
				LOCKSTATS_UNPREFETCH(lockname);		\
				_LOCK_UNPREFETCH(lockname)

#endif

#ifdef USE_TS
#include "ts_lock.h"
#define _LOCKDEC(lockname)		TS_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		TS_LOCKINIT(lockname)
#define _LOCK(lockname)			TS_LOCK(lockname)
#define _UNLOCK(lockname)		TS_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		TS_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		TS_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	TS_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	TS_UNPREFETCH(lockname)
#endif USE_TS

#ifdef USE_TTS
#include "tts_lock.h"
#define _LOCKDEC(lockname)		TTS_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		TTS_LOCKINIT(lockname)
#define _LOCK(lockname)			TTS_LOCK(lockname)
#define _UNLOCK(lockname)		TTS_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		TTS_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		TTS_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	TTS_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	TTS_UNPREFETCH(lockname)
#endif USE_TTS

#ifdef USE_TSBACK
#include "tsback_lock.h"
#define _LOCKDEC(lockname)		TSBACK_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		TSBACK_LOCKINIT(lockname)
#define _LOCK(lockname)			TSBACK_LOCK(lockname)
#define _UNLOCK(lockname)		TSBACK_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		TSBACK_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		TSBACK_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	TSBACK_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	TSBACK_UNPREFETCH(lockname)
#endif USE_TSBACK

#ifdef USE_TTSBACK
#include "ttsback_lock.h"
#define _LOCKDEC(lockname)		TTSBACK_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		TTSBACK_LOCKINIT(lockname)
#define _LOCK(lockname)			TTSBACK_LOCK(lockname)
#define _UNLOCK(lockname)		TTSBACK_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		TTSBACK_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		TTSBACK_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	TTSBACK_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	TTSBACK_UNPREFETCH(lockname)
#endif USE_TTSBACK

#ifdef USE_TSBACKR
#include "tsbackr_lock.h"
#define _LOCKDEC(lockname)		TSBACKR_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		TSBACKR_LOCKINIT(lockname)
#define _LOCK(lockname)			TSBACKR_LOCK(lockname)
#define _UNLOCK(lockname)		TSBACKR_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		TSBACKR_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		TSBACKR_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	TSBACKR_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	TSBACKR_UNPREFETCH(lockname)
#endif USE_TSBACKR

#ifdef USE_TTSBACKR
#include "ttsbackr_lock.h"
#define _LOCKDEC(lockname)		TTSBACKR_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		TTSBACKR_LOCKINIT(lockname)
#define _LOCK(lockname)			TTSBACKR_LOCK(lockname)
#define _UNLOCK(lockname)		TTSBACKR_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		TTSBACKR_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		TTSBACKR_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	TTSBACKR_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	TTSBACKR_UNPREFETCH(lockname)
#endif USE_TTSBACKR

#ifdef USE_MCS
#include "mcs_lock.h"
#define _LOCKDEC(lockname)		MCS_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		MCS_LOCKINIT(lockname)
#define _LOCK(lockname)			MCS_LOCK(lockname)
#define _UNLOCK(lockname)		MCS_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		MCS_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		MCS_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	MCS_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	MCS_UNPREFETCH(lockname)
#endif USE_MCS

#ifdef USE_LH
#include "lh_lock.h"
#define _LOCKDEC(lockname)		LH_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		LH_LOCKINIT(lockname)
#define _LOCK(lockname)			LH_LOCK(lockname)
#define _UNLOCK(lockname)		LH_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		LH_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		LH_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	LH_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	LH_UNPREFETCH(lockname)
#endif USE_LH

#ifdef USE_M
#include "m_lock.h"
#define _LOCKDEC(lockname)		M_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		M_LOCKINIT(lockname)
#define _LOCK(lockname)			M_LOCK(lockname)
#define _UNLOCK(lockname)		M_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		M_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		M_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	M_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	M_UNPREFETCH(lockname)
#endif USE_M

#ifdef USE_REACTIVE
#include "reactive_lock.h"
#define _LOCKDEC(lockname)		REACTIVE_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		REACTIVE_LOCKINIT(lockname)
#define _LOCK(lockname)			REACTIVE_LOCK(lockname)
#define _UNLOCK(lockname)		REACTIVE_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		REACTIVE_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		REACTIVE_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	REACTIVE_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	REACTIVE_UNPREFETCH(lockname)
#endif USE_REACTIVE

#ifdef USE_REACTIVE2
#include "reactive2_lock.h"
#define _LOCKDEC(lockname)		REACTIVE2_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		REACTIVE2_LOCKINIT(lockname)
#define _LOCK(lockname)			REACTIVE2_LOCK(lockname)
#define _UNLOCK(lockname)		REACTIVE2_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		REACTIVE2_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		REACTIVE2_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	REACTIVE2_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	REACTIVE2_UNPREFETCH(lockname)
#endif USE_REACTIVE2

#ifdef USE_QOLB
#include "qolb_lock.h"
#define _LOCKDEC(lockname)		QOLB_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		QOLB_LOCKINIT(lockname)
#define _LOCK(lockname)			QOLB_LOCK(lockname)
#define _UNLOCK(lockname)		QOLB_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		QOLB_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		QOLB_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	QOLB_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	QOLB_UNPREFETCH(lockname)
#endif USE_QOLB

#ifdef USE_WITH
#include "with_lock.h"
#define _LOCKDEC(lockname)		WITH_LOCKDEC(lockname)
#define _LOCKINIT(lockname)		WITH_LOCKINIT(lockname)
#define _LOCK(lockname)			WITH_LOCK(lockname)
#define _UNLOCK(lockname)		WITH_UNLOCK(lockname)
#define _LOCK_ENTRY(lockname)		WITH_LOCK_ENTRY(lockname)
#define _UNLOCK_ENTRY(lockname)		WITH_UNLOCK_ENTRY(lockname)
#define _LOCK_PREFETCH(lockname)	WITH_PREFETCH(lockname)
#define _LOCK_UNPREFETCH(lockname)	WITH_UNPREFETCH(lockname)
#endif USE_WITH

#endif /* _lock_h */
