/*
 * qolb_lock.h
 *
 * Copyright (c) 1996 by Jim Goodman
 *
 * ALL RIGHTS RESERVED.
 *
 * This source file is part of the SCI protocol module for the Wisconsin
 * Wind Tunnel, written by Alain K\"agi.
 *
 * ----------------------------------------------------------------------------
 *
 * Qolb lock.
 */

#ifndef _qolb_lock_h
#define _qolb_lock_h

/*
 * Declare a lock.
 */
#define QOLB_LOCKDEC(lockname)		volatile int lockname

/*
 * Initialize a lock.
 */
#define QOLB_LOCKINIT(lockname)		_qolb_initialize(&(lockname))

/*
 * Initialize a lock, using sbrk instead of malloc.  This is for malloc.
 * For QOLB there's no need to allocate anything, may be different for
 * other locks.
 */
#define QOLB_SBRK_LOCKINIT(lockname)	QOLB_LOCKINIT(lockname)

/*
 * Calls.
 */
#define QOLB_LOCK(lockname)		_qolb_acquire_acq(&(lockname))
#define QOLB_UNLOCK(lockname)		_qolb_release_rel(&(lockname))

#define QOLB_LOCK_ENTRY(lockname)	_qolb_acquire(&(lockname))
#define QOLB_UNLOCK_ENTRY(lockname)	_qolb_release(&(lockname))

#define QOLB_PREFETCH(lockname)		_qolb_prefetch(&(lockname))
#define QOLB_UNPREFETCH(lockname)	_qolb_unprefetch(&(lockname))



/*
 * Details follow.
 */
INLINE void
_qolb_initialize(void *targetVA)
{
  _qolb_deqolb_i(targetVA);
  *(int *)targetVA = 0;
  _qolb_deqolb_i(targetVA);
}

INLINE void
_qolb_acquire_acq(void *targetVA)
{
  do {
    while(!_qolb_enqolb(targetVA))
      ; /* Wait for line to become local */
  } while(wwt_swap_acq(targetVA, 1));
}

INLINE void
_qolb_release_rel(void *targetVA)
{
  asm volatile ("");
  *(int *)targetVA = 0;
  _qolb_deqolb_rel(targetVA);
}

/*
 * Qolb/entry consistency.
 */
INLINE void
_qolb_acquire(void *targetVA)
{
  do {
    while(!_qolb_enqolb(targetVA))
      ; /* Wait for line to become local */
  } while(wwt_swap(targetVA, 1));
}

INLINE void
_qolb_release(void *targetVA)
{
  asm volatile ("");
  *(int *)targetVA = 0;
  _qolb_deqolb(targetVA);
}

#endif _qolb_lock_h
